<?php
if (!defined('ABSPATH')) exit;
include_once(ABSPATH.'wp-admin/includes/plugin.php');


if( class_exists('EM_Addon_Base') ) {
	new EM_Addon_More();
}


#===============================================
# 	Construct this Section.
#===============================================
class EM_Addon_More {

	public function __construct(){
		add_action('em_addons_menu', array($this, 'create_menu_item'), 100);
	}


	#===============================================
	# 	Create Admin Sub Menu under Events.
	#===============================================
	public static function create_menu_item() {
		global $config;
		add_submenu_page(
			'stonehenge_em_addons',
			__('More Add-ons', $config['text']),
			__('More...', $config['text']),
			'install_plugins',
			'stonehenge_addons',
			array($this, 'show_this_section')
		);
	}


	public static function define_addons() {
		$addons = array(
			array(
				'id' 		=> 'events-manager-email-users',
				'name'		=> 'EM – Email Users',
				'text' 		=> 'Send emails to bookings of a specific event, based on their booking status. <br><b>Pro version has been released!</b>',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-osm',
				'name'		=> 'EM – OpenStreetMaps',
				'text' 		=> 'OpenStreetMaps for Events Manager. Replace Google Maps with OpenStreetMap.<br>0% Google, 100% Open Source.',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-slider',
				'name'		=> 'EM – Events / Locations Slider',
				'text' 		=> 'Create a dynamic Slider (carousel or fade) with EM arguments you already know.',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-move-bookings',
				'name'		=> 'EM – Move Bookings',
				'text' 		=> 'Easily move EM Bookings from one Event to another, using a simple dropdown select box.',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-cancellation',
				'name'		=> 'EM – Event Cancellation',
				'text' 		=> 'Adds an "Event Cancelled" status to your EM event and auto-sends "rejected" status emails.',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-ongoing-events',
				'name'		=> 'EM – Ongoing Events',
				'text' 		=> 'Organizing multiday events with just one booking / sign-up? This add-on is perfect for courses, classes, seminars, etc.',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-waiting-list',
				'name'		=> 'EM – Waiting Lists',
				'text' 		=> 'Easily create a waiting list for fully booked events and show it automatically on your front-end when applicable.',
				'active' 	=> false,
			),
			array(
				'id' 		=> 'stonehenge-em-gateway-selector',
				'name'		=> 'EM Pro – Gateway Selector',
				'text' 		=> 'Easily select  your payment gateway(s) per individual single event with a simple checkbox!',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-em-mollie',
				'name'		=> 'EM Pro – Mollie Gateway',
				'text' 		=> 'Add iDEAL, Apple Pay, Bancontact, SOFORT, GiroPay, EPS, SEPA, Klarna and more using Mollie.',
				'active' 	=> true,
			),
			array(
				'id' 		=> 'stonehenge-price-converter',
				'name'		=> 'Simple Price Converter',
				'text' 		=> 'Display your product price in any of the 50 supported currencies (used in 70 countries) using daily exchange rates. (Powered by ExchangeRate-API.com)',
				'active' 	=> false,
			),
		);
		return $addons;
	}


	public static function process_addons() {
		global $config;
		$addons = self::define_addons();

		ob_start();
		foreach( $addons as $addon ) {
			$image 	= plugins_url('images/'. $addon['id'] .'.jpg', __FILE__);
			$url 	= 'https://wordpress.org/plugins/'. $addon['id'] .'/';
			$file 	= "{$addon['id']}/{$addon['id']}.php";
			$name  	= $addon['name'];

			// Use links or not?
			if( isset($addon['active']) && $addon['active'] === true ) {
				echo "<table class='addon-card'><tr>";
				echo "<td width='100px'><a href='{$url}' target='_blank'><img src='{$image}' class='icon' alt='{$name}'></a></td>";
				echo "<td><h4><a href='{$url}' target='_blank'>{$addon['name']}</a></h4><p>{$addon['text']}</p></td>";
				echo "</tr><tr><td colspan='2'>";
				if( !is_plugin_active( $file) ) {
					$install = sprintf(__('Install %s'), 'Plugin');
					echo "<button class='stonehenge-button' style='float:right;'><a href='{$url}' target='_blank'>{$install}</a></button>";
				} else {
					echo '<span class="stonehenge-success" style="float:right;">'. __('Plugin <strong>activated</strong>.') .'</span>';
				}
				echo "</td></tr></table>";
			} else {
				echo "<table class='addon-card'><tr>";
				echo "<td width='100px'><img src='{$image}' class='icon' alt='{$name}'></td>";
				echo "<td><h4>{$addon['name']}</h4><p>{$addon['text']}</p></td>";
				echo "</tr><tr><td colspan='2'><span class='pro' style='float:right;'><b>" . __('Coming Soon', $config['text']) ."</b></span></td></tr></table>";
			}
		}
		$output = ob_get_clean();
		return $output;
	}



	#===============================================
	# 	Render the Premium Features Page.
	#===============================================
	public static function show_this_section() {
		global $config;
		wp_enqueue_style('stonehenge-css', plugins_url('stonehenge.min.css', __FILE__), null, '', 'screen');
		?>
		<div class="wrap">
			<h1><?php echo __('Even More Goodies!', $config['text']); ?> &#x1F603;</h1>
			<div id="poststuff">
				<div class="inside">
					<div id="postbox-container" class="postbox-container stonehenge-settings">
						<div class="meta-box-sortables ui-sortable" id="normal-sortables">
							<div class="postbox" id="more">
								<h3 class="hndle"></h3>
								<div class="inside">
									<p>Here is an overview of other plugins and add-ons that I have created. Feel free to check them all out! :-)<br>
										Missing anything? Let me know!
									</p>
									<?php echo self::process_addons(); ?>
								</div>
							</div>
						</div>
					</div>
				</div>
			</div>
		</div>
		<?php
	}

} // End class.
