<?php
// Exit if accessed directly
if (!defined('ABSPATH')) exit;


class Stonehenge_EM_OSM_Maps {

	public function __construct() {
		add_filter('em_event_output_placeholder', array( 'Stonehenge_EM_OSM_Maps', 'replace_location_map_placeholder'), 1, 3);
		add_filter('em_location_output_placeholder', array( 'Stonehenge_EM_OSM_Maps', 'replace_location_map_placeholder'), 1, 3);

		remove_shortcode('locations_map');
		add_shortcode('locations_map', array('Stonehenge_EM_OSM_Maps', 'output_locations_map_shortcode'));

		remove_shortcode('events_map');
		add_shortcode('events_map', array('Stonehenge_EM_OSM_Maps', 'output_events_map_shortcode'));

		// Deprecated, but here for backward compatibility.
		remove_shortcode('locations-map');
		add_shortcode('locations-map', array('Stonehenge_EM_OSM_Maps', 'output_locations_map_shortcode'));
		remove_shortcode('events-map');
		add_shortcode('events-map', array('Stonehenge_EM_OSM_Maps', 'output_events_map_shortcode'));
	}


	#===============================================
	# 	Replace #_LOCATIONMAP
	#===============================================
	public static function replace_location_map_placeholder( $replacement, $input, $result ) {
		// $input is used so both $EM_Location and $EM_Event can be processed.
		if( $result === '#_LOCATIONMAP' ) {
			$replacement = ($input->location_id) ? Stonehenge_EM_OSM_Maps::output_single_map_placeholder($input->location_id) : '';
		}
		return $replacement;
	}


	#===============================================
	# 	Output for #_LOCATIONMAP
	#===============================================
	public static function output_single_map_placeholder($input) {
		// $input is the Location ID of $EM_Location or $EM_Event.
		global $EM_Event, $EM_Location;

		$EM_Location 	= new EM_Location($input);

		// Get the right information of the correct Object.
		if( is_object($EM_Event) ) {
			$id 			= 'L' . $EM_Event->location_id . 'E' . $EM_Event->event_id;
			$balloon 		= trim(preg_replace('/\s\s+/', '<br>', get_option('dbem_location_baloon_format')));
			$balloon 		= $EM_Event->output($balloon);
		} else {
			$id 			= 'L' . $EM_Location->location_id;
			$balloon 		= trim(preg_replace('/\s\s+/', '<br>', get_option('dbem_map_text_format')));
			$balloon 		= $EM_Location->output($balloon);
		}
		$balloon 		= addslashes($balloon);
		$longitude		= $EM_Location->location_longitude;
		$latitude		= $EM_Location->location_latitude;

		$width 			= get_option('dbem_map_default_width') ? get_option('dbem_map_default_width') : '400px';
		$width 			= preg_match('/(px)|%/', $width) ? $width : $width.'px';
		$height 		= get_option('dbem_map_default_height') ? get_option('dbem_map_default_height') : '300px';
		$height 		= preg_match('/(px)|%/', $height) ? $height:$height.'px';

		// Start output.
		ob_start();
		?>
		<div id="em-osm-single-map-container-<?php echo $id; ?>" class="em-osm-container">
			<link rel="stylesheet" href="<?php echo  plugins_url('inc/em-osm-style.min.css', __FILE__); ?>">
			<script src="<?php echo plugins_url('inc/em-osm-leaflet.min.js', __FILE__); ?>"></script>
			<div id="map<?php echo $id; ?>" class="em-osm-map" style="width: <?php echo $width; ?>; height: <?php echo $height; ?>;"></div>
			<?php echo self::fetch_saved_map_options(); ?>
			<script>
				var Lat = <?php echo $latitude; ?>;
				var Lng = <?php echo $longitude; ?>;
				var balloon = '<?php echo $balloon; ?>';

				var map<?php echo $id; ?> = L.map('map<?php echo $id;?>', {
				    center: [Lat, Lng],
				    minZoom: 0,
				    maxZoom: 19,
				    zoom: zoomLevel,
				    zoomControl: zoomButtons,
				    zoomSnap: 0.25,
					scrollWheelZoom: mobileZoom,
					dragging: mobileDrag,
				});

				L.tileLayer(mapTiles, {
				    attribution: '&copy; <a href=\"https://www.openstreetmap.org/copyright\" target=\"_blank\">OpenStreetMap</a>',
				}).addTo(map<?php echo $id; ?>);

				var marker<?php echo $id;?> = L.marker([Lat, Lng], { icon: mapIcon }, ).addTo(map<?php echo $id;?>).bindPopup(balloon).openPopup();

				if( showFullscreen == 'yes' ) 	{ map<?php echo $id; ?>.addControl(new L.Control.Fullscreen({ position: 'topright', })); }
				if( showScale == 'yes' ) 		{ L.control.scale().addTo(map<?php echo $id; ?>); }

				setTimeout(function(){ map<?php echo $id; ?>.invalidateSize()}, 400);
			</script>
		</div>
		<?php
		$output = ob_get_clean();
		$output = EM_Addon::minify_js($output);
		return $output;
	}


	#===============================================
	# 	New output for Admin map (back- & front-end).
	#===============================================
	public static function output_single_admin_map($EM_Location) {
		$saved 			= get_option('stonehenge_em_osm');
		$api 			= !empty($saved['api']) ? $saved['api'] : '';
		$locale 		= strtolower(substr( get_bloginfo ( 'language' ), 0, 2 )); // Get the first 2 letters of the current language
		$location_id 	= $EM_Location->location_id;
		$balloon 		= !empty($location_id) ? addslashes("<b>{$EM_Location->location_name}</b><br>{$EM_Location->location_address}, {$EM_Location->location_town}") : '';
		$defCountry 	= get_option('dbem_location_default_country');
		ob_start();
		?>
		<div id="em-osm-admin-map-container" class="em-osm-container">
			<link rel="stylesheet" href="<?php echo  plugins_url('inc/em-osm-style.min.css', __FILE__); ?>">
			<script src="<?php echo plugins_url('inc/em-osm-leaflet.min.js', __FILE__); ?>"></script>
			<div id="em-osm-map" style="width:400px; height:300px;" class="em-osm-map"></div>
			<?php echo self::fetch_saved_map_options(); ?>
			<script>
				var defCountry 		= '<?php echo $defCountry; ?>';
				var locID 			= '<?php echo $location_id; ?>';
				var Lat  			= document.getElementById("location-latitude").value;
				var Lng 			= document.getElementById("location-longitude").value;
				var balloon 		= '<?php echo $balloon; ?>';
				var noLocation		= '<?php echo __('No Location','events-manager'); ?>';

				if( Lat == 0 && Lng == 0) { tempZoom = '1'; balloon	= noLocation; }
				else { tempZoom = zoomLevel; }

				var map = L.map('em-osm-map', {
				    center: [Lat, Lng],
				    minZoom: 0,
				    maxZoom: 19,
				    zoom: tempZoom,
				    zoomControl: zoomButtons,
				    zoomSnap: 0.25,
					scrollWheelZoom: mobileZoom,
					dragging: mobileDrag,
				});

				L.tileLayer(mapTiles, { attribution: '&copy; <a href=\"https://www.openstreetmap.org/copyright\" target=\"_blank\">OpenStreetMap</a>', }).addTo(map);

				var marker = L.marker([Lat, Lng], {icon: mapIcon}, ).addTo(map).bindPopup(balloon).openPopup();

				if( showFullscreen == 'yes' ) 	{ map.addControl(new L.Control.Fullscreen({ position: 'topright', })); }
				if( showScale == 'yes' ) 		{ L.control.scale().addTo(map); }

				setTimeout(function(){ map.invalidateSize()}, 400);

/* Minified version of em-osm-location.js */
function apiSearch(){geoAddress(document.getElementById("location-address").value+", "+document.getElementById("location-town").value+", "+document.getElementById("location-postcode").value+", "+document.getElementById("location-state").value)}function geoAddress(e){jQuery.ajax({url:"https://api.opencagedata.com/geocode/v1/json",method:"GET",data:{key:'<?php echo $api; ?>',q:e,no_annotations:1,add_request:1,pretty:1,language:'<?php echo $locale; ?>' },dataType:"json",statusCode:{200:function(e){var t=e.results[0];console.log(t);var o=t.formatted,a=t.components;if(a.city&&jQuery("#location-town").val(a.city),a.town&&jQuery("#location-town").val(a.town),a.village&&jQuery("#location-town").val(a.village),a.state&&jQuery("#location-state").val(a.state),a.county&&jQuery("#location-region").val(a.county),a.postcode&&jQuery("#location-postcode").val(a.postcode),a.country_code){var l=a.country_code.toUpperCase();jQuery("#location-country").val(l)}var n=t.geometry.lat,i=t.geometry.lng,c=new L.LatLng(n,i);jQuery("#location-latitude").val(n),jQuery("#location-longitude").val(i),marker.setLatLng(c).bindPopup(o).openPopup(),map.setView(c,zoomLevel)},402:function(){console.log("hit free-trial daily limit"),console.log("become a customer: https://opencagedata.com/pricing")}}})}jQuery("#no-location").change(function(){if(jQuery("#no-location").is(":checked")){if(jQuery("#em-location-data").hide(),jQuery("#location-id").val(""),jQuery("#location-latitude").val(""),jQuery("#location-longitude").val(""),jQuery("#location-select-id").length){jQuery("#location-select-id option[value='0']").prop("selected","selected");jQuery("#location-select-id option:selected").text()}}else jQuery("#em-location-data").show(),jQuery("#location-id").val(locID),jQuery("#location-latitude").val(Lat),jQuery("#location-longitude").val(Lng)}).trigger("change"),jQuery("#location-select-id").change(function(){var e=jQuery("#location-select-id option:selected").attr("title").split(","),t=e[0],o=e[1],a=new L.LatLng(t,o),l=jQuery("#location-select-id option:selected").text(),n=jQuery("#location-select-id option:selected").val();jQuery("#location-latitude").val(t),jQuery("#location-longitude").val(o),jQuery("#location-id").val(n),"0"==n?(jQuery("#location-latitude").val(""),jQuery("#location-longitude").val(""),jQuery("#location-id").val(""),marker.setLatLng([0,0]).bindPopup(l),map.setView([0,0],1)):(marker.setLatLng(a).bindPopup(l).openPopup(),map.setView(a,zoomLevel))}).trigger("change"),jQuery("#location-name").length>0||jQuery("#location-select-id").length?marker.dragging.disable():(marker.dragging.enable(),marker.on("dragend",function(e){jQuery("#location-latitude").val(marker.getLatLng().lat),jQuery("#location-longitude").val(marker.getLatLng().lng)}));
			</script>
		</div>
		<?php
		$output = ob_get_clean();
		$output = EM_Addon::minify_js($output);
		return $output;
	}


	#===============================================
	# 	Output for [locations_map]
	#===============================================
	public static function output_locations_map_shortcode($args) {
		global $EM_Event, $EM_Location, $EM_Object;
		$saved 		= get_option('stonehenge_em_osm');

		// Create fallback to prevent errors.
		if(empty($args)) {
			$args = array();
		}
		$width 		= @$args['width'] ? @$args['width'] : get_option('dbem_map_default_width');
		$width 		= preg_match('/(px)|%/', $width) ? $width : $width.'px';
		$height 	= @$args['height'] ? @$args['height'] : get_option('dbem_map_default_height');
		$height 	= preg_match('/(px)|%/', $height) ? $height : $height.'px';
		$return 	= '';

		$EM_Locations 	= EM_Locations::get( $args, $count=false );
		$marker 		= array();
		$lats 			= array();
		$lngs 			= array();

		if( count($EM_Locations) === 0 ) {
			echo '<p><em>'. esc_html__('No locations found', 'stonehenge-em-osm').'</em></p>';
			return;
		}

		// Process individual locations.
		foreach( $EM_Locations as $EM_Location ) {
			$latitude 		= $EM_Location->location_latitude;
			$longitude 		= $EM_Location->location_longitude;
			$balloon 		= trim(preg_replace('/\s\s+/', '<br>', get_option('dbem_map_text_format')));
			$balloon 		= $EM_Location->output($balloon);
			$balloon 		= addslashes($balloon);
			$marker[] 		= "[\"{$balloon}\", {$latitude}, {$longitude}]";
			$lats[] 		= $latitude;
			$lngs[] 		= $longitude;
		}
		$markers 	= implode(", ", $marker);
		$high_lat 	= max($lats);
		$high_lng 	= max($lngs);
		$low_lat 	= min($lats);
		$low_lng 	= min($lngs);
		$avg_lat 	= array_sum($lats)/count($lats);
		$avg_lng 	= array_sum($lngs)/count($lngs);

		$mapbounds = (count($EM_Locations) === 1) ? sprintf('setView([%s, %s], %s);', $high_lat, $high_lng, $saved['zoom']) : sprintf('fitBounds([[%s, %s], [%s, %s]], {padding: [10, 10]});', $high_lat, $high_lng, $low_lat, $low_lng);

		// Start the output.
		ob_start();
		$id = rand(1,100);
		?>
		<div id="em-osm-locations-map-container-<?php echo $id; ?>" class="em-osm-container">
			<link rel="stylesheet" href="<?php echo  plugins_url('inc/em-osm-style.min.css', __FILE__); ?>">
			<script src="<?php echo plugins_url('inc/em-osm-leaflet.min.js', __FILE__); ?>"></script>
			<div id="em-osm-map-<?php echo $id; ?>" class="em-osm-map-multiple" style="width: <?php echo $width;?>; height: <?php echo $height; ?>;"></div>
			<?php echo self::fetch_saved_map_options(); ?>
			<script>
				var locations = [<?php echo $markers; ?>];
				var map<?php echo $id; ?> = L.map('em-osm-map-<?php echo $id; ?>', {
				    center: [<?php echo $avg_lat; ?>, <?php echo $avg_lng; ?>],
				    minZoom: 0,
				    maxZoom: 19,
				    zoom: zoomLevel,
				    zoomControl: zoomButtons,
				    zoomSnap: 0.25,
					scrollWheelZoom: mobileZoom,
					dragging: mobileDrag,
				});

				L.tileLayer(mapTiles, {
					attribution: '&copy; <a href=\"https://www.openstreetmap.org/copyright\" target=\"_blank\">OpenStreetMap</a>',
				}).addTo(map<?php echo $id; ?>);

				map<?php echo $id; ?>.<?php echo $mapbounds; ?>

				for (var i = 0; i < locations.length; i++) {
					marker = new L.marker([locations[i][1],locations[i][2]], {icon: mapIcon},).addTo(map<?php echo $id; ?>).bindPopup(locations[i][0]);
				}
				function clickZoom(e) {
					map<?php echo $id; ?>.setView(e.target.getLatLng(),zoomLevel);
				}

				if( showFullscreen == 'yes' ) 	{ map<?php echo $id; ?>.addControl(new L.Control.Fullscreen({ position: 'topright', })); }
				if( showScale == 'yes' ) 		{ L.control.scale().addTo(map<?php echo $id; ?>); }

				setTimeout(function(){ map<?php echo $id; ?>.invalidateSize()}, 400);
			</script>
		</div>
		<?php
		$script = ob_get_clean();
		$script = EM_Addon::minify_js($script);
		return $script;
	}


	#===============================================
	# 	Output for [events_map]
	#===============================================
	public static function output_events_map_shortcode($args) {
		$saved 		= get_option('stonehenge_em_osm');

		// Create fallback to prevent errors.
		if(empty($args)) {
			$args = array();
		}
		$width 			= @$args['width'] ? @$args['width'] : get_option('dbem_map_default_width');
		$width 			= preg_match('/(px)|%/', $width) ? $width : $width.'px';
		$height 		= @$args['height'] ? @$args['height'] : get_option('dbem_map_default_height');
		$height 		= preg_match('/(px)|%/', $height) ? $height : $height.'px';

		$EM_Events 		= EM_Events::get( $args, $count = false );
		$location_ids 	= array();
		foreach( $EM_Events as $EM_Event ) {
			// Filter out Events without  location.
			if( '0' != $EM_Event->location_id ) {
				$location_ids[] = $EM_Event->location_id;
			}
		}

		// Clean the array.
		$location_ids 	= array_unique( $location_ids );

		if( count($location_ids) === '0' ) {
			echo '<p><em>'. esc_html__('No events found', 'stonehenge-em-osm').'</em></p>';
			return;
		}
		$marker 	= array();
		$lats 		= array();
		$lngs 		= array();
		foreach( $location_ids as $location_id ) {
			$EM_Location 	= new EM_Location($location_id);
			$latitude 		= $EM_Location->location_latitude;
			$longitude 		= $EM_Location->location_longitude;
			$balloon 		= trim(preg_replace('/\s\s+/', '<br>', get_option('dbem_map_text_format')));
			$balloon 		= addslashes( $EM_Location->output( $balloon ) );
			$marker[] 		= "[\"{$balloon}\", {$latitude}, {$longitude}]";
			$lats[] 		= $latitude;
			$lngs[] 		= $longitude;
		}
		$markers 	= implode(", ", $marker);
		$high_lat 	= max($lats);
		$high_lng 	= max($lngs);
		$low_lat 	= min($lats);
		$low_lng 	= min($lngs);
		$avg_lat 	= array_sum($lats)/count($lats);
		$avg_lng 	= array_sum($lngs)/count($lngs);

		$mapbounds = (count($location_ids) === 1) ? sprintf('setView([%s, %s], %s);', $high_lat, $high_lng, $saved['zoom']) : sprintf('fitBounds([[%s, %s], [%s, %s]], {padding: [10, 10]});', $high_lat, $high_lng, $low_lat, $low_lng);

		// Start the output.
		ob_start();
		$id = rand(1,100);
		?>
		<div id="em-osm-events-map-container-<?php echo $id; ?>" class="em-osm-container">
			<link rel="stylesheet" href="<?php echo  plugins_url('inc/em-osm-style.min.css', __FILE__); ?>">
			<script src="<?php echo plugins_url('inc/em-osm-leaflet.min.js', __FILE__); ?>"></script>
			<div id="em-osm-map-<?php echo $id; ?>" class="em-osm-map-multiple" style="width: <?php echo $width;?>; height: <?php echo $height; ?>;"></div>
			<?php echo self::fetch_saved_map_options(); ?>
			<script>
				var locations = [<?php echo $markers; ?>];
				var map<?php echo $id; ?> = L.map('em-osm-map-<?php echo $id; ?>', {
				    center: [<?php echo $avg_lat; ?>, <?php echo $avg_lng; ?>],
				    minZoom: 0,
				    maxZoom: 19,
				    zoom: zoomLevel,
				    zoomControl: zoomButtons,
				    zoomSnap: 0.25,
					scrollWheelZoom: mobileZoom,
					dragging: mobileDrag,
				});

				L.tileLayer(mapTiles, {
					attribution: '&copy; <a href=\"https://www.openstreetmap.org/copyright\" target=\"_blank\">OpenStreetMap</a>',
				}).addTo(map<?php echo $id; ?>);

				map<?php echo $id; ?>.<?php echo $mapbounds; ?>

				for (var i = 0; i < locations.length; i++) {
					marker = new L.marker([locations[i][1],locations[i][2]], {icon: mapIcon},).addTo(map<?php echo $id; ?>).bindPopup(locations[i][0]);
				}
				function clickZoom(e) {
					map<?php echo $id; ?>.setView(e.target.getLatLng(),zoomLevel);
				}

				if( showFullscreen == 'yes' ) 	{ map<?php echo $id; ?>.addControl(new L.Control.Fullscreen({ position: 'topright', })); }
				if( showScale == 'yes' ) 		{ L.control.scale().addTo(map<?php echo $id; ?>); }

				setTimeout(function(){ map<?php echo $id; ?>.invalidateSize()}, 400);
			</script>
		</div>
		<?php
		$script = ob_get_clean();
		$script = EM_Addon::minify_js($script);
		return $script;
	}


	#===============================================
	# 	Plugin options for all maps.
	#===============================================
	public static function fetch_saved_map_options() {
		$saved 			= get_option('stonehenge_em_osm');
		$default 		= "//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png";
		$mapTiles 		= !empty($saved['type']) ? $saved['type'] : $default;
		$markerColor	= !empty($saved['marker']) ? $saved['marker'] : 'blue';
		$markerURL		= plugins_url('inc/markers/marker-icon-2x-'. $markerColor . '.png', __FILE__);
		$markerShadow	= plugins_url('inc/markers/marker-shadow.png', __FILE__);
		$zoomLevel 		= !empty($saved['zoom']) ? $saved['zoom'] : '15';
		$zoomControls	= (@$saved['zoomcontrols'] != 'no') ? 'true' : 'false';
		$showFullscreen	= (@$saved['fullscreen'] != 'no') ? 'yes' : 'no';
		$showScale 		= (@$saved['scale'] != 'no' ? 'yes' : 'no' );

		ob_start();
		?>
		<script>
			var mapTiles = '<?php echo $mapTiles; ?>';
			var zoomLevel = <?php echo $zoomLevel; ?>;
			var zoomButtons = '<?php echo $zoomControls; ?>';
			var showFullscreen = '<?php echo $showFullscreen; ?>';
			var showScale = '<?php echo $showScale; ?>';

			var mapIcon = new L.Icon({
			    iconUrl: '<?php echo $markerURL; ?>',
			    shadowUrl: '<?php echo $markerShadow; ?>',
			    iconSize: [25, 41],
			    iconAnchor: [12, 41],
			    popupAnchor: [1, -34],
			    shadowSize: [41, 41]
			});

			// Check for Mobile
			if(L.Browser.mobile) {
				var mobileDrag = false;
				var mobileZoom = false;
			}
			else {
				var mobileDrag = true;
				var mobileZoom = true;
			}
		</script>
		<?php
		$script = ob_get_clean();
		$script = EM_Addon::minify_js($script);
		return $script;
	}

} // End class.
